Exception: Rich::ColorParseError
- Inherits:
-
StandardError
- Object
- StandardError
- Rich::ColorParseError
- Defined in:
- lib/rich/color.rb
Overview
Error raised when a color definition cannot be parsed
Error raised when a color definition cannot be parsed